package opmanager

import (
	"fmt"
	"regexp"
	"strconv"
)

// Represents the Interface Traffic from OpManager.
// OpManager returns strings like "480.147 K (4.8%)" (where the unit is optional)
type InterfaceTraffic struct {
	Traffic          float64 // 480.147
	Unit             string  // K
	UnitFormated     string  // kbits - Unit for openITCOCKPIT
	OitcUnit         string  // hardcoded to bps for openITCOCKPIT
	OitcValue        float64 // 480147 bits/s bc openITCOCKPIT always wants the smallest possible unit (to enable auto scale with the unit scaler)
	Percentage       float64 // 4.8
	ParsedSuccessful bool    // Determine if the traffic string got parsed successfully
}

func ParseInterfaceTraffic(traffic string) (InterfaceTraffic, error) {
	if traffic == "NA" || traffic == "" {
		// Just return a empty struct where all values are zero
		return InterfaceTraffic{}, nil
	}

	// Parse Traffic Strings like
	// - 206.489  (0.0%)
	// - 480.147 K (4.8%)
	// - 2.761 M (2.1%)
	// - 1.923 G (9.62%)
	// This Regex is stolen from ChatGPT
	// 1. `(\d+(\.\d+)?)`: This part captures the numeric value.
	// 2. `\s*`: This part matches zero or more whitespace characters (spaces).
	// 3. `(K|M|G)?`: This part captures an optional unit, which can be K, M, or G. The `?` at the end makes the unit part optional.
	// 4. `\s*`: This part again matches zero or more whitespace characters.
	// 5. `\((\d+(\.\d+)?)%\)`: This part captures the percentage value enclosed in parentheses.
	pattern := regexp.MustCompile(`(\d+(\.\d+)?)\s*(K|M|G|T)?\s*\((\d+(\.\d+)?)%\)`)

	match := pattern.FindStringSubmatch(traffic)
	if match != nil {

		traffic, _ := strconv.ParseFloat(match[1], 64)
		unit := match[3]
		if unit == "" {
			unit = "B"
		}
		percentage, _ := strconv.ParseFloat(match[4], 64)

		unitFormated, factorToBits := getMetricUnitForOpenITCOCKPIT(unit)

		return InterfaceTraffic{
			Traffic:          traffic,
			Unit:             unit,
			UnitFormated:     unitFormated,
			OitcUnit:         "bps",
			OitcValue:        traffic * float64(factorToBits),
			Percentage:       percentage,
			ParsedSuccessful: true,
		}, nil
	}

	return InterfaceTraffic{}, fmt.Errorf("could not parse traffic string '%v'", traffic)
}

// Converts the Traffic Unit from OpManger into Metric-Units openITCOCKPIT and Grafana can understand
func getMetricUnitForOpenITCOCKPIT(OpManagerUnit string) (string, int64) {
	// https://github.com/it-novum/openITCOCKPIT/blob/development/src/itnovum/openITCOCKPIT/Grafana/GrafanaTargetUnit.php#L112

	switch OpManagerUnit {
	case "T":
		return "tbits", 1000 * 1000 * 1000 * 1000 // Terabits/sec (Tbit/s)

	case "G":
		return "gbits", 1000 * 1000 * 1000 // Gigabits/sec (Gbit/s)

	case "M":
		return "mbits", 1000 * 1000 // Megabits/sec (Mbit/s)

	case "K":
		return "kbits", 1000 // Kilobits/sec (kbit/s)
	}

	return "bps", 0 // Bits/sec
}
