package prtg

// PRTG API Docs
// https://www.paessler.com/manuals/prtg/http_api
// Supported Output Columns ("columns=" Parameter)
// https://www.paessler.com/manuals/prtg/multiple_object_property_or_status
//
// PRTG Date Formats
// https://kb.paessler.com/en/topic/1313-how-do-i-translate-prtg-timestamp-values-format-to-normal-time-format#reply-2883
// https://stackoverflow.com/a/70389039/11885414
// https://github.com/qax-os/excelize/blob/master/date.go
// https://pkg.go.dev/github.com/ganqierwu/excelize/v2#ExcelDateToTime
//
// We use HTTP API v1, because V2 is not ready for production:
// > The PRTG API v2 is currently in the alpha phase which means that it is not yet stable and not feature-complete. If you already want to use the PRTG API on your production system, we recommend that you use the PRTG API (v1). For more information, click here.
// https://www.paessler.com/support/prtg/api/v2/overview#toc_5

// CURL Examples API V1
// curl -k "https://172.16.165.19/api/table.json?content=sensors&columns=objid,type,name,active,downtime,downtimetime,sensor,lastcheck,device,probe,status,message,lastup&apitoken=BV3PKNYJHAUEQIR7UTCZWFYUN7FKNTO62DFBUFQZ5M======" | jq .

// Die PRTG API gibt ein Feld treesize zurück. Den Wert davon müssen wir in count= setzten damit wir alle Sensoren über die alte V1 API abholen können

// Legenden besagen, das man count=* setzten kann, um alle Werte zurück zu bekommen
// https://github.com/lordmilko/PrtgAPI/issues/367#issuecomment-1815924496

// The Config struct needs to match the json structure from the openITCOCKPIT database
type Config struct {
	Verbose              bool
	ApiUrl               string `json:"api_url"`      // API Endpoint. E.g.: https://172.16.165.19/
	ApiUser              string `json:"api_user"`     // User name for the API E.g.: "root"
	ApiPassword          string `json:"api_password"` // Password for the given user
	ApiToken             string `json:"api_token"`    // API Token instead of username + password (recommended)
	UseProxy             int    `json:"use_proxy"`    // Use a HTTP Proxy (0 or 1)
	ProxyAddress         string // Address of the HTTP Proxy to use (not from DB JSON string)
	IgnoreSslCertificate int    `json:"ignore_ssl_certificate"` // Do not verify the TLS Certificate for HTTPS connections (0 or 1)
	PollingInterval      int64  `json:"polling_interval"`       // Interval in seconds how often the PRTG API should be polled
	IncludeChannels      int    `json:"include_channels"`       // Synchronize channel metrics from external PRTG system (may cause many API requests).
}

// Map PRTG states into Naemon states
// Source: https://www.paessler.com/manuals/prtg/multiple_object_property_or_status
const (
	PRTG_NONE                 int64 = 0
	PRTG_UNKNOWN              int64 = 1
	PRTG_SCANNING             int64 = 2 // Alternative wording "COLLECTING"
	PRTG_UP                   int64 = 3
	PRTG_WARNING              int64 = 4
	PRTG_DOWN                 int64 = 5
	PRTG_NO_PROBE             int64 = 6
	PRTG_PAUSED_BY_USER       int64 = 7
	PRTG_PAUSED_BY_DEPENDENCY int64 = 8
	PRTG_PAUSED_BY_SCHEDULE   int64 = 9
	PRTG_UNUSUAL              int64 = 10
	PRTG_NOT_LICENSE          int64 = 11 // Alternative wording "PAUSED_BY_LICENSE"
	PRTG_PAUSED_UNTIL         int64 = 12
	PRTG_DOWN_ACKNOWLEDGED    int64 = 13
	PRTG_DOWN_PARTIAL         int64 = 14
)

// PRTG Channel IDs
// https://www.paessler.com/manuals/prtg/channel_settings
const (
	PRTG_CHANNEL_SUM      int64 = -1
	PRTG_CHANNEL_DOWNTIME int64 = -4
	PRTG_CHANNEL_COVERAGE int64 = -2 // PRTG internally only
	PRTG_CHANNEL_ERROR    int64 = -3 // PRTG internally only
)
